@@ -1,7 +1,7 @@
 #-*- coding:utf-8 -*-
 
 #  Pybik -- A 3 dimensional magic cube game.
-#  Copyright © 2009-2011  B. Clausius <barcc@gmx.de>
+#  Copyright © 2009-2012  B. Clausius <barcc@gmx.de>
 #
 #  This program is free software: you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -16,23 +16,35 @@
 #  You should have received a copy of the GNU General Public License
 #  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from __future__ import print_function
+from __future__ import print_function, division, unicode_literals
 
-__all__ = ['error', 'debug', 'debug_func', 'DEBUG_LEVEL',
-            'DEBUG_FUNC', 'DEBUG_MSG', 'DEBUG_ROTATE',
-            'DEBUG_DRAW', 'DEBUG_KEYS', 'DEBUG_TEST', 'DEBUG_PICK']
+import sys
+
+
+DEBUG = False
+DEBUG_FUNC = DEBUG_MSG = DEBUG_RAND = DEBUG_ROTATE = False
+DEBUG_DRAW = DEBUG_KEYS = DEBUG_TEST = DEBUG_PICK = False
+DEBUG_FPS = DEBUG_VFPS = DEBUG_NOLABEL = DEBUG_NOBEVEL = DEBUG_INNER = False
+DEBUG_ALG = False
+
+__all__ = ['error', 'debug', 'debug_func', 'DEBUG',
+          ] + [__n for __n in dir(sys.modules[__name__]) if __n.startswith('DEBUG_')]
 
 def error(*args, **kwargs):
     print('ERROR:', *args, **kwargs)
     
 debug = lambda *args, **kwargs: None
 debug_func = lambda x: x
-DEBUG_LEVEL = 0
-DEBUG_FUNC = DEBUG_MSG = DEBUG_ROTATE = DEBUG_DRAW = DEBUG_KEYS = DEBUG_TEST = DEBUG_PICK = False
 
-def install(module, debug_modes):
-    for m, v in debug_modes.items():
-        setattr(module, m, v)
+def set_flags(debug_flags):
+    module = sys.modules[__name__]
+    module.DEBUG = True
+    
+    for flag in debug_flags:
+        setattr(module, 'DEBUG_' + flag.upper(), True)
+        
+    if module.DEBUG_VFPS:
+        module.DEBUG_FPS = True
     
     if module.DEBUG_FUNC:
         def _debug_pre_func(func, *args, **kwargs):
@@ -42,18 +54,22 @@ def install(module, debug_modes):
                 if len(arg) > maxlen:
                     return arg[:maxlen]
                 return arg
-            print('%s--%s' % ('  |'*debug_func.indent, func.func_name))
+            try:
+                func_name = func.func_name
+            except AttributeError:
+                func_name = func.__name__
+            print('%s--%s' % ('  |'*debug_func.indent, func_name))
             debug_func.indent += 1
             for arg in args:
                 try:
                     print('%s: %s' % ('  |'*debug_func.indent, short_arg(arg)))
-                except:
-                    pass 
+                except Exception:
+                    pass
             for kw, arg in kwargs:
                 try:
                     print('%s: %s=%s' % ('  |'*debug_func.indent, kw, short_arg(arg)))
-                except:
-                    pass 
+                except Exception:
+                    pass
         def debug_func(func):
             def ret_func(*args, **kwargs):
                 _debug_pre_func(func, *args, **kwargs)
@@ -67,11 +83,17 @@ def install(module, debug_modes):
                 finally:
                     debug_func.indent -= 1
                     print('  |'*debug_func.indent + "--'")
-            ret_func.__dict__ = func.__dict__
+            try:
+                ret_func.__dict__ = func.__dict__
+            except AttributeError:
+                pass
             ret_func.__doc__ = func.__doc__
             ret_func.__module__ = func.__module__
             ret_func.__name__ = func.__name__
-            ret_func.func_defaults = func.func_defaults
+            try:
+                ret_func.func_defaults = func.func_defaults
+            except AttributeError:
+                pass
             return ret_func
         debug_func.indent = 0
         debug_func.last_exc = None
