@@ -16,132 +16,136 @@
 #  You should have received a copy of the GNU General Public License
 #  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-
 # Ported from GNUbik
 # Original filename: guile-hooks.c
 # Original copyright and license: 2004  Dale Mellor, GPL3+
 
+from __future__ import print_function, division, unicode_literals
 
 import os
 import sys
 import imp
-import re
-from collections import namedtuple
+from collections import namedtuple, OrderedDict
 
-from .debug import *
+from .debug import *    # pylint: disable=W0401, W0614
 from . import config
-from . import move_queue
+from .moves import MoveQueue
+from . import model
+
+FILE_VERSION = '1.1'
 
 
-class PluginHelper (object):
+class PluginState (object):
     def __init__(self, app):
-        self.app = app
-        # When a script runs,  the first cube movement that is requested by the script
-        # must flush the move queue after the place of current insertion; further move
-        # requests must be appended to the end of the move queue. This variable is used
-        # to track the first request.
-        self.moved = False
-        plugin_funcs = [
-                ('register_script',  self.plugin_register_script),
-                ("cube_state",       self.plugin_cube_state),
-                ("rotate_animated",  self.plugin_rotate_animated),
-                ('rotate_flubrd',    self.plugin_rotate_flubrd),
-                ("run_moves",        self.plugin_run_moves),
-                ("error_dialog",     self.plugin_error_dialog),
-                ('random',  lambda x:app.current_movement.current_cube_state.rand.randrange(x)),
-            ]
-        module = sys.modules['pybikplugin'] = type(sys)('pybikplugin')
-        for name, func in plugin_funcs:
-            setattr(module, name, func)
+        self.initial_state = None
+        self.current_state = app.gamestate.current_cube_state.copy()
+        self.moves = MoveQueue()
+        self.error_dialog = app.error_dialog
+        self.set_progress = app.set_progress
+        self.end_progress = app.end_progress
+        
+    def add_moves(self, moves):
+        for move in moves:
+            self.current_state.rotate_slice(move)
+            self.moves.push(move)
             
-    def __del__(self):
-        self.app = None
-        
-    def call(self, func):
-        if not callable(func):
-            return False
-        self.moved = False
-        func()
-        return self.moved
-        
-    def plugin_register_script(self, path, func):
-        self.scripts.append((path, func))
-        
-    def plugin_cube_state(self):
-        '''Function callable from plugin which returns a structure
-        reflecting the current state of the cube.'''
-        return self.app.current_movement.current_cube_state.copy()
-        
-    def _start_moves_if_first(self):
-        '''The first time a script makes a move on the cube,  the move_queue must be
-        truncated to the current place,  and the place is marked so that the viewer
-        can rewind the effects of the script. This script performs the necessary
-        preparations.'''
-        if not self.moved:
-            self.moved = 1
-            self.app.current_movement.all_moves.truncate()
-            self.app.current_movement.all_moves.mark_current(True)
-            
-    def plugin_rotate_animated(self, moves):
-        '''Function which, when called from plugins, causes one
-        side of the cube to rotate on-screen.'''
-        self._start_moves_if_first()
-        for maxis, mslice, mdir in moves:
-            data = move_queue.MoveData(maxis, mslice, mdir)
-            self.app.current_movement.all_moves.push(data)
-            
-    def plugin_rotate_flubrd(self, code, size):
-        self._start_moves_if_first()
+    def add_flubrd(self, code):
         #TODO: here flubrd should be used, not the one in all_moves,
         # the converter in all_moves may be configurable in the future
-        moves = []
-        for move_data, unused_pos in self.app.current_movement.all_moves.parse_iter(code, len(code), size):
-            moves.append((move_data.axis, move_data.slice, move_data.dir))
-        return moves
+        for move_data, unused_pos in self.moves.parse_iter(code, len(code), self.current_state.model):
+            self.current_state.rotate_slice(move_data)
+            
+    def random(self, count=-1):
+        self.current_state.set_solved()
+        self.current_state.randomize(count)
+        self.initial_state = self.current_state.copy()
+        self.moves.reset()
         
-    def plugin_run_moves(self):
-        '''Function allowing a script to apply all its moves in one go to the cube,
-        without creating animations on the display.'''
-        self.app.current_movement.do_fast_forward()
+    def test_model(self, models, show_error=True):
+        cube = self.current_state
+        for unused_modelstr, Model, sizes, exp in models:
+            if Model == '*':
+                return True
+            if Model is None:
+                continue
+            if cube.type != Model.type:
+                continue
+            sizedict = {}
+            for size1, size2 in zip(sizes, cube.model.sizes):
+                if size1 is None:
+                    continue
+                if type(size1) is int:
+                    if size1 != size2:
+                        break
+                else:
+                    sizedict[size1] = size2
+            else:
+                try:
+                    if exp is None or eval(exp, {}, sizedict):
+                        return True
+                except ValueError:
+                    continue
+        if not show_error:
+            return False
+        models = [model_[0] for model_ in models if model_[1] is not None]
+        if not models:
+            self.error_dialog(_('This algorithm does not work for any model.\n'))
+        elif len(models) == 1:
+            self.error_dialog(_('This algorithm only works for:\n') + '  ' + models[0])
+        else:
+            self.error_dialog(
+                    _('This algorithm only works for:\n') +
+                    '\n'.join([' • ' + m for m in models])
+                )
+        return False
         
-    def plugin_error_dialog(self, message):
-        '''Function to allow a plugin to display a message to the user.'''
-        self.app.error_dialog(message)
         
-    def _load_py_plugin(self, dirname, modulename):
-        file_, path, desc = imp.find_module(modulename, [dirname])
-        try:
-            module = imp.load_module(modulename, file_, path, desc)
-            debug('    import', modulename)
-        except Exception as e:
-            debug(e)
-        finally:
-            if file_:
-                file_.close()
+class PluginHelper (object):
+    def __init__(self):
+        self.scripts = OrderedDict()
+        self.error_messages = []
         
-    def _load_plugins_from_directory(self, dirname):
-        if os.path.isdir(dirname):
-            debug('Plugins from', dirname)
-            sys.path.insert(0, dirname)
-            for filename in sorted(os.listdir(dirname), key=str.lower):
-                debug('  script:', filename)
-                name, ext = os.path.splitext(filename)
-                if ext == '.py':
-                    self._load_py_plugin(dirname, name)
-                elif ext == '.script':
-                    self._load_text_plugin(os.path.join(dirname, filename))
+    def call(self, app, index):
+        plugin_state = PluginState(app)
+        all_models = []
+        for models, func in self.scripts.values()[index]:
+            if plugin_state.test_model(models, show_error=False):
+                func(plugin_state)
+                break
+            all_models += models
+        else:
+            plugin_state.test_model(all_models, show_error=True)
+        return plugin_state.initial_state, plugin_state.moves
+        
+    def load_plugins_from_directory(self, dirname):
+        if not os.path.isdir(dirname):
+            debug('Plugins path does not exist:', dirname)
+            return
+        debug('Loading Plugins from', dirname)
+        sys.path.insert(0, dirname)
+        for filename in sorted(os.listdir(dirname), key=unicode.lower):
+            unused_name, ext = os.path.splitext(filename)
+            if ext != '.algorithm':
+                continue
+            debug('  algorithm:', filename)
+            try:
+                self.load_text_plugin(os.path.join(dirname, filename))
+            except Exception as e:      # pylint: disable=W0703
+                self.error_messages.append('Error loading {}:\n{}'.format(os.path.basename(filename), e))
+                sys.excepthook(*sys.exc_info())
         
     def load_plugins(self):
         '''This function initializes the plugins for us, and once the plugins have
         all been imported, it returns the requested menu structure to the caller.'''
-        self.scripts = []
-        self._load_plugins_from_directory(config.SCRIPT_DIR)
-        script_dir = os.path.join(config.get_data_home(), 'scripts')
-        self._load_plugins_from_directory(script_dir)
+        self.scripts.clear()
+        del self.error_messages[:]
+        self.load_plugins_from_directory(config.SCRIPT_DIR)
+        self.load_plugins_from_directory(config.USER_SCRIPT_DIR)
         debug('  found', len(self.scripts))
-        return self.scripts
+        return [(path, i) for i, path in enumerate(self.scripts.keys())]
         
-    def _load_text_plugin(self, filename):
+    @staticmethod
+    def parse_text_plugin(filename):
         with open(filename) as fp:
             lines = fp.readlines()
         paras = []
@@ -150,7 +154,7 @@ class PluginHelper (object):
         para = {}
         key = None
         for line in lines:
-            line = line.rstrip()
+            line = unicode(line, 'utf-8').rstrip()
             if not line:
                 # end para
                 if para:
@@ -170,38 +174,47 @@ class PluginHelper (object):
                 para[key] = [value] if value else []
         if para:
             paras.append(para)
+        return paras
         
-        if not paras:
-            return
-            
-        # evaluate header
-        header = paras[0].copy()
-        value = header.get('File-Version', [])
-        if value != ['1.0']:
-            debug('Wrong file version:', '\n'.join(value))
-            return
-        models = header.get('Model', [])
-        if not models:
-            debug('Field "Model" not specified')
-            return
-        def split_model(model):
-            type_ = model.split(' ', 1)
-            if len(type_) != 2 or type_[0] != 'Cube':
-                return 'unknown', 'unknown', None
-            type_, size = type_
+    @staticmethod
+    def eval_model(modelstrings):
+        model_infos = []
+        for modelstr in modelstrings:
             try:
-                size = int(size)
-            except ValueError:
-                return 'unknown', 'unknown', None
-            return model, type_, size
-        models = [split_model(m) for m in models]
+                model_info = model.from_string(modelstr)
+            except ValueError as e:
+                debug('Error in model %s:' % modelstr, e)
+            else:
+                model_infos.append(model_info)
+        return model_infos
+        
+    @classmethod
+    def eval_header(cls, header):
+        value = header.get('File-Version', [])
+        if len(value) == 0:
+            value = FILE_VERSION
+            debug('No file version found, assume version', value)
+        elif len(value) == 1:
+            value = value[0]
+        else:
+            debug('Multiple version numbers found:\n ', '\n  '.join(value))
+            value = value[0]
+            debug('assume version', value)
+        if value != FILE_VERSION:
+            debug('Wrong file version:', value)
+            return None, None
+        models = header.get('Model', [])
+        models = cls.eval_model(models)
+        
         ref_blocks = header.get('Ref-Blocks', None)
         if ref_blocks:
             ref_blocks = ' '.join(ref_blocks).split()
-            
-        # evaluate solutions
+        return models, ref_blocks
+        
+    @classmethod
+    def eval_paras(cls, models, ref_blocks, paras):
         for para in paras:
-            value = para.get('Path', [])
+            value = para.get('Path', ())
             if len(value) != 1 or not value[0]:
                 debug('    skip Path:', value)
                 para.clear()
@@ -224,6 +237,9 @@ class PluginHelper (object):
             value = [split_path(v)[1] for v in value]
             para['Depends'] = value
             
+            value = para.get('Model', None)
+            para['Model'] = models if value is None else cls.eval_model(value)
+            
             value = para.get('Ref-Blocks', None)
             if value:
                 value = ' '.join(value).split()
@@ -231,7 +247,7 @@ class PluginHelper (object):
             
             value = para.get('Solution', None)
             def split_solution(value):
-                value = value.split('#',1)[0]
+                value = value.split('#', 1)[0]
                 value = value.split(',', 1)
                 if len(value) != 2:
                     return None
@@ -243,78 +259,109 @@ class PluginHelper (object):
                 value = [split_solution(v) for v in value if v]
             para['Solution'] = value
             
+            value = para.get('Moves', None) or None
+            if value is not None:
+                value = ' '.join(value)
+            para['Moves'] = value
+            
+            value = para.get('Module', None) or None
+            if value is not None:
+                value = value[0]
+            para['Module'] = value
+            
+    def load_text_plugin(self, filename):
+        paras = self.parse_text_plugin(filename)
+        if not paras:
+            return
+            
+        # evaluate header
+        models, ref_blocks = self.eval_header(paras[0])
+        if models is None:
+            return
+            
+        # evaluate solutions
+        self.eval_paras(models, ref_blocks, paras)
+        
         scripts = []
         for para in paras:
             if not para:
                 continue
             sep, path = para['Path']
-            params = namedtuple('ScriptParams', 'depends ref_blocks solution scripts models')(
-                                depends=para['Depends'],
+            models = para['Model']
+            depends = para['Depends']
+            solution = para['Solution']
+            moves = para['Moves']
+            module = para['Module']
+            if depends or solution is not None:
+                if moves is not None:
+                    debug('    solution can not have moves:', path)
+                if module is not None:
+                    debug('    solution can not have a module field:', path)
+                params = ScriptParams(
+                                depends=depends,
                                 ref_blocks=para['Ref-Blocks'],
-                                solution=para['Solution'],
+                                solution=solution,
                                 scripts=scripts,
                                 models=models,
+                                sep_path=(sep, path),
                             )
-            scripts.append((path, sep!='@', ScriptFactory(self, params)))
+                scripts.append((path, sep!='@', models, ScriptFactory(self, params)))
+            elif moves is not None:
+                if module is not None:
+                    debug('    Moves can not have a module field:', path)
+                def play_moves(moves, models):
+                    def func(game):
+                        if game.test_model(models):
+                            game.add_flubrd(moves)
+                    return func
+                scripts.append((path, True, models, play_moves(moves, models)))
+            elif module is not None:
+                dirname = os.path.dirname(filename)
+                modulename, funcname = module.split(' ', 1)
+                mfile, mpath, mdesc = imp.find_module(modulename, [dirname])
+                moduleobj = imp.load_module(modulename, mfile, mpath, mdesc)
+                modulefunc = getattr(moduleobj, funcname)
+                scripts.append((path, True, models, modulefunc))
+            else:
+                debug('    skip Path without algorithm:', path)
             
-        self.scripts += [(path, func) for path, visible, func in scripts if visible]
+        for path, visible, models, func in scripts:
+            if visible:
+                funclist = self.scripts.setdefault(path, [])
+                funclist.append((models, func))
         
         
+ScriptParams = namedtuple('ScriptParams', 'depends ref_blocks solution scripts models sep_path')
+
 class ScriptFactory (object):
     def __init__(self, plugin, params):
         self.solved_face_colors = {}
         self.plugin = plugin
         self.params = params
         
-    def __call__(self):
-        cube = self.plugin.plugin_cube_state()
-        for model, type_, dimension in self.params.models:
-            if cube.type == type_ and cube.dimension == dimension:
-                break
-        else:
-            self.plugin.plugin_error_dialog(
-                            _('This script only works for:') +
-                            ''.join('\n    '+m for m,t,d in self.params.models))
+    def __call__(self, plugin_state):
+        if not plugin_state.test_model(self.params.models):
             return
             
         depends = list(reversed(self.params.depends))
-        scripts = {path: func for path, visible, func in self.params.scripts}
+        scripts = {path: func for path, visible, models, func in self.params.scripts}
         for depend in depends:
             instance = scripts[depend]
             depends += instance.params.depends
         for depend in reversed(depends):
             instance = scripts[depend]
-            self.execute(cube, instance.params.ref_blocks, instance.params.solution)
-        self.execute(cube, self.params.ref_blocks, self.params.solution)
+            self.execute(plugin_state, instance.params)
+        self.execute(plugin_state, self.params)
         
-    def get_color(self, cube, face, block):
-        facenum = 'udlrfb'.index(face)
-        f, l, u = 1, 1, 1
-        for match in re.finditer(r'(.)(\d*)', block):
-            blockface, blockslice = match.groups()
-            blockslicenum = int(blockslice) if blockslice else 1
-            if blockface == 'u':   u = blockslicenum - 1
-            elif blockface == 'd': u = cube.dimension - blockslicenum
-            elif blockface == 'l': l = blockslicenum - 1
-            elif blockface == 'r': l = cube.dimension - blockslicenum
-            elif blockface == 'f': f = blockslicenum - 1
-            elif blockface == 'b': f = cube.dimension - blockslicenum
-        if face in 'ud': index = f + cube.dimension * l
-        if face in 'lr': index = f + cube.dimension * u
-        if face in 'fb': index = l + cube.dimension * u
-        
-        return cube.faces[facenum][index]
-        
-    @debug_func
-    def test_face(self, cube, position, condition, face):
+    def test_face(self, cube, blocksym, condition, face):
         try:
             color2 = self.solved_face_colors[condition[face]]
         except KeyError:
             return True
-        color1 = self.get_color(cube, position[face], position)
+        blocknum = cube.model.block_symbolic_to_block_index(blocksym)
+        color1 = cube.get_colorsymbol(blocknum, blocksym[face])
         return color1 == color2
         
-    @debug_func
     def test_basic_condition(self, cube, position, condition):
         assert len(position) == len(condition)
         for i in xrange(len(position)):
@@ -322,13 +369,13 @@ class ScriptFactory (object):
                 return False
         return True
         
-    def opposite(self, face):
+    @staticmethod
+    def opposite(face):
         return {  'f': 'b', 'b': 'f',
                   'l': 'r', 'r': 'l',
                   'u': 'd', 'd': 'u',
                }[face]
         
-    @debug_func
     def test_pattern_condition(self, cube, position, condition):
         if '?' in condition:
             conditions = (condition.replace('?', face, 1)
@@ -339,11 +386,11 @@ class ScriptFactory (object):
         else:
             return self.test_basic_condition(cube, position, condition)
             
-    def rotated_conditions(self, condition):
+    @staticmethod
+    def rotated_conditions(condition):
         for i in range(len(condition)):
             yield condition[i:] + condition[:i]
         
-    @debug_func
     def test_prefix_condition(self, cube, position, condition):
         if condition.startswith('!*'):
             return not self.test_or_conditions(cube, position, self.rotated_conditions(condition[2:]))
@@ -358,50 +405,53 @@ class ScriptFactory (object):
         else:
             return self.test_pattern_condition(cube, position, condition)
         
-    @debug_func
     def test_or_conditions(self, cube, position, conditions):
         for prefix_cond in conditions:
             if self.test_prefix_condition(cube, position, prefix_cond):
                 return True
         return False
         
-    @debug_func
     def test_and_conditions(self, cube, conditions):
         for position, or_cond in conditions:
             if not self.test_or_conditions(cube, position, or_cond.split('|')):
                 return False
         return True
         
-    def execute(self, cube, ref_blocks, rules):
+    def execute(self, plugin_state, params):
+        rules = params.solution
         if rules is None:
             return
+        cube = plugin_state.current_state
         count = 0
         pos = 0
         while pos < len(rules):
             self.solved_face_colors.clear()
-            for block in ref_blocks:
+            for block in params.ref_blocks:
+                blocknum = cube.model.block_symbolic_to_block_index(block)
                 for face in block:
-                    self.solved_face_colors[face] = self.get_color(cube, face, block)
+                    self.solved_face_colors[face] = cube.get_colorsymbol(blocknum, face)
             conditions, moves = rules[pos]
             if self.test_and_conditions(cube, conditions):
-                debug('positive: {}. {!r}, {!r}'.format(
-                                pos, ' '.join('='.join(c) for c in conditions), moves))
+                if DEBUG_ALG:
+                    print('{}: accept: {:2}. {}, {}'.format(
+                                params.sep_path[0].join(params.sep_path[1]),
+                                pos+1,
+                                ' '.join('='.join(c) for c in conditions),
+                                moves))
                 if moves == '@@solved':
                     return
                 if count > 4 * len(rules): # this value is just guessed
-                    self.plugin.plugin_error_dialog(
+                    plugin_state.error_dialog(
                         'An infinite loop was detected. '
                         'This is probably an error in the solution.')
                     return
                 count += 1
                 
-                moves_ = self.plugin.plugin_rotate_flubrd(moves, cube.dimension)
-                for move in moves_:
-                    cube._rotate_slice(*move)
+                plugin_state.add_flubrd(moves)
                 pos = 0
             else:
                 pos += 1
-        self.plugin.plugin_error_dialog(
+        plugin_state.error_dialog(
             'No matching rules found. '
             'This is probably an error in the solution.')
         
